# -*- coding: utf-8 -*-
# Copyright 2014, Red Hat, Inc.
# License: GNU General Public License version 2 or later

import libtaskotron.bodhi_utils as bodhi
from libtaskotron.koji_utils import KojiClient
from libtaskotron.directives import BaseDirective

from libtaskotron.logger import log
from libtaskotron.exceptions import TaskotronDirectiveError

directive_class = 'BodhiDirective'

class BodhiDirective(BaseDirective):
    """The bodhi directive interfaces with Bodhi to facilitate
    various bodhi actions.

    format: "bodhi:
               action: [download]
               <command args>"

    Valid Commands:

    download
      Downloads rpms of builds from update specified by its name or id
      format: "bodhi:
                 action: download
                 bodhi_id: <name or id of update>
                 arch=<desired arch>"
    """
    def __init__(self, bodhi_api=None, koji_session=None):

        if bodhi_api:
            self.bodhi_api = bodhi_api
        else:
            self.bodhi_api = bodhi.BodhiUtils()

        if koji_session:
            self.koji_session = koji_session
        else:
            self.koji_session = KojiClient()

    def action_download(self, update, arches, workdir):

        res = self.bodhi_api.query_update(update)

        if res is None:
            raise TaskotronDirectiveError("Update with id '%s' wasn't found"
                                    % update)

        nvrs = [build['nvr'] for build in res['builds']]

        downloaded_rpms = []

        for nvr in nvrs:
            res = self.koji_session.get_nvr_rpms(nvr, workdir, arches)
            downloaded_rpms.extend(res)

        return downloaded_rpms

    def process(self, input_data, env_data):
        output_data = {}

        valid_actions = ['download']

        action = input_data['action']

        if action not in valid_actions:
            raise TaskotronDirectiveError('%s is not a valid command for bodhi directive'
                                 % action)

        if action == 'download':
            if 'bodhi_id' not in input_data or 'arch' not in input_data:
                detected_args = ', '.join(input_data.keys())
                raise TaskotronDirectiveError("The bodhi directive 'download'"
                   " requires both 'bodhi_id' and"
                   " 'arch' arguments. Detected arguments: %s" % detected_args)

            workdir = env_data['workdir']
            update = input_data['bodhi_id']
            if input_data['arch'] == 'all':
                arches = ['i386', 'x86_64', 'noarch', 'armhfp']
            else:
                arches = [arch.strip() for arch in input_data['arch'].split(',')]
            if 'noarch' not in arches and arches != ['src']:
                arches.append('noarch')

            log.info("getting rpms for update %s (%s) and downloading to %s",
                      update, arches, workdir)

            output_data['downloaded_rpms'] = self.action_download(update, arches, workdir)

        return output_data
