from libtaskotron.koji_utils import KojiClient
from libtaskotron.logger import log
from libtaskotron.directives import BaseDirective
from libtaskotron.exceptions import TaskotronDirectiveError

directive_class = 'KojiDirective'

class KojiDirective(BaseDirective):
    """The koji directive interfaces with Koji to facilitate various koji
    actions.

    format: "koji: target_dir=<target_dir> command=[download, download_tag] <command args>

    Valid Commands:

    download
      Downloads the envr specified as "envr"
      format:
        "koji: command=download envr=<envr to download> arch=<desired arch>
    download_tag
      Downloads rpms tagged by "tag"
      format:
        "koji: command=download_tag tag=<tag to download> arch=<desired arch>
    """

    def __init__(self, koji_session=None):
        super(KojiDirective, self).__init__()
        if koji_session is None:
            self.koji = KojiClient(koji_session)
        else:
            self.koji = koji_session

    def process(self, input_data, env_data):
        valid_actions = ['download', 'download_tag']

        output_data = {}

        action = input_data['action']

        if not action in valid_actions:
            raise TaskotronDirectiveError('%s is not a valid command for koji '
                                          'helper' % action)

        if 'arch' not in input_data:
            detected_args = ', '.join(input_data.keys())
            raise TaskotronDirectiveError(
                "The koji directive requires 'arch' as an argument. Detected"
                "arguments: %s" % detected_args)

        if not 'target_dir' in input_data:
            self.target_dir = env_data['workdir']
        else:
            self.target_dir = input_data['target_dir']

        self.arches = [arch.strip() for arch in input_data['arch'].split(',')]

        # https://phab.qadevel.cloud.fedoraproject.org/T155
        if 'noarch' not in self.arches:
            self.arches.append('noarch')

        if action == 'download':
            if 'envr' not in input_data:
                detected_args = ', '.join(input_data.keys())
                raise TaskotronDirectiveError(
                    "The koji directive requires 'envr' for the 'download'"
                    "action. Detected arguments: %s" % detected_args)

            envr = input_data['envr']

            log.info("getting koji builds for %s (%s) and downloading to %s",
                     envr, str(self.arches), self.target_dir)
            output_data['downloaded_rpms'] = self.koji.get_nvr_rpms(envr,
                                                                self.target_dir,
                                                                self.arches)
        elif action == 'download_tag':
            if 'tag' not in input_data:
                detected_args = ', '.join(input_data.keys())
                raise TaskotronDirectiveError(
                    "The koji directive requires 'tag' for the 'download_tag'"
                    "action. Detected arguments: %s" % detected_args)

            tag = input_data['tag']

            log.info("getting koji builds for tag '%s', arches '%s' and "
                     "downloading to %s", tag, str(self.arches), self.target_dir)
            output_data['downloaded_rpms'] = self.koji.get_tagged_rpms(tag,
                                                                self.target_dir,
                                                                self.arches)

        return output_data
