# Copyright 2014, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#       Josef Skladanka <jskladan@redhat.com>

from libtaskotron.directives import BaseDirective

from libtaskotron import check
from libtaskotron import config

from libtaskotron.exceptions import TaskotronDirectiveError, TaskotronValueError
from libtaskotron.logger import log

import resultsdb_api

directive_class = 'ResultsdbDirective'

class ResultsdbDirective(BaseDirective):
    """The resultsdb directive interfaces with ResultsDB to store results.

    format: "resultsdb: results=<TAP13>
    """

    def __init__(self, resultsdb = None):
        self.resultsdb = resultsdb

        conf = config.get_config()
        self.masterurl = conf.taskotron_master
        self.task_stepname = conf.buildbot_task_step

        if self.resultsdb is None:
            self.resultsdb = resultsdb_api.ResultsDBapi(conf.resultsdb_server)

    def ensure_testcase_exists(self, name):
        """ Make sure that the testcase exists in resultsdb, otherwise create
        the testcase using a dummy url as a reference

        :param str name: name of the testcase to check for
        """

        try:
            testcase = self.resultsdb.get_testcase(name)
            return
        except resultsdb_api.ResultsDBapiException, e:
            if e.message != 'Testcase not found':
                raise e

        # since the testcase doesn't exist, create it with a dummy value for url
        # it can be updated later when it's not blocking results reporting
        dummyurl = 'http://faketestcasesRus.com/%s' % name
        self.resultsdb.create_testcase(name, dummyurl)

    def create_resultsdb_job(self, name, refurl=None):
        """ Create a job in resultsdb for reporting results against

        :param str name: name of job to report against
        :returns: dict of rendered json results
        """

        # this is completly hacky for now since the runner doesn't have a
        # good way to get the actual refurl. This will be addressed in T122 and
        # T145

        url = 'http://not-a-real-domain.com/noworky'
        if refurl is not None:
            url = refurl

        jobdata = self.resultsdb.create_job(url, status='SCHEDULED', name=name)
        self.resultsdb.update_job(id=jobdata['id'], status='RUNNING')

        return jobdata

    def complete_resultsdb_job(self, jobid):
        """ Change the resultsdb job to a status of 'COMPLETED', indicating
        that the reporting is complete.

        :param int jobid: The resultsdb jobid to be modified
        :returns: json representation of returned data from the resultsdb instance
        """
        return self.resultsdb.update_job(jobid, status='COMPLETED')

    def parse_jobid(self, jobid):
        """ Parse the incoming jobid which should either be '-1' to indicate
        that dummy values should be used or in the form of <builder>/<buildid>.

        If the passed in format is invalid, no exceptions will be raised but a
        warning message will be emitted to logs

        :param str jobid: jobid from runner
        :returns: (builder, buildid)
        """
        if jobid != "-1":
            try:
                builder, buildid = jobid.split('/')
                return (builder, buildid)
            except ValueError, e:
                log.debug(e)
                log.warning(
                    'Invalid jobid format detected, resetting to default '
                    'value. jobid must be in builder/buildid format, '
                    'found %s' % jobid)

        return "default", "-1"

    def process(self, input_data, env_data):

        conf = config.get_config()
        if not (conf.reporting_enabled and conf.report_to_resultsdb):
            log.info("Reporting to ResultsDB is disabled.")
            return

        #TODO: Replace with proper location and adjust exc. text
        # we're creating the jobid in the directive for now, so adjust the check

        if 'checkname' not in env_data:
            raise TaskotronDirectiveError("The resultsdb directive requires "\
                    "resultsdb_job_id and checkname.")

        try:
            check_details = check.import_TAP(input_data['results'])
        except TaskotronValueError as e:
            raise TaskotronDirectiveError("Failed to load 'results': %s"  % e.message)

        builder, buildid = self.parse_jobid(env_data['jobid'])

        # for now, we're creating the resultsdb job at reporting time
        # the job needs to be 'RUNNING' in order to append any results
        job_url = '%s/builders/%s/builds/%s' % (self.masterurl, builder, buildid)
        job_data = self.create_resultsdb_job(env_data['checkname'], refurl=job_url)

        self.ensure_testcase_exists(env_data['checkname'])

        output = []

        for detail in check_details:
            log_url = '%s/steps/%s/logs/stdio' % (job_url, self.task_stepname)
            try:
                result = self.resultsdb.create_result(
                        job_id = job_data['id'],
                        testcase_name = env_data['checkname'],
                        outcome = detail.outcome,
                        summary = detail.summary or None,
                        log_url = log_url,
                        item = detail.item,
                        type = detail.report_type,
                        **detail.keyvals)
                output.append(result)

            except resultsdb_api.ResultsDBapiException, e:
                log.error(e)
                log.error("Failed to store ResulsDB: `%s` `%s` `%s`",
                            detail.item, env_data['checkname'], detail.outcome)

        # move job to completed
        self.complete_resultsdb_job(job_data['id'])

        return output

