# -*- coding: utf-8 -*-
# Copyright 2014, Red Hat, Inc.
# License: GNU General Public License version 2 or later

''' Utility methods related to Koji '''

import collections
import koji
import hawkey

from libtaskotron import file_utils
from libtaskotron.logger import log
from libtaskotron import exceptions as exc
from libtaskotron import rpm_utils
from libtaskotron import config


class KojiClient(object):
    '''Helper Koji methods.

    :ivar koji.ClientSession session: Koji client session
    '''
    def __init__(self, koji_session=None):
        '''Create a new KojiClient

        :param koji_session: an existing Koji session instance or ``None`` if
                             you want a new default session to be created
        :type koji_session: :class:`koji.ClientSession`
        '''
        self.session = (koji_session or
            koji.ClientSession(config.get_config().koji_url))


    def nvr_to_urls(self, nvr, arches=None, debuginfo=False, src=True):
        '''Get list of URLs for RPMs corresponding to a build.

        :param str nvr: build NVR
        :param arches: restrict the arches of builds to provide URLs for. Note:
                       If basearch ``i386`` is in the list, ``i686`` arch is
                       automatically added (since that's the arch Koji API uses).
        '''
        log.info('Querying Koji for a list of RPMS for: %s', nvr)

        # add i686 arch if i386 is present in arches
        if arches and 'i386' in arches and 'i686' not in arches:
            arches.append('i686')

        info = self.session.getBuild(nvr)
        if info is None:
            raise exc.TaskotronRemoteError(
                "No build information found for %s" % nvr)

        baseurl = '/'.join((config.get_config().pkg_url, info['package_name'],
                            info['version'], info['release']))

        rpms = self.session.listRPMs(buildID=info['id'], arches=arches)
        if not debuginfo:
            rpms = [r for r in rpms if not r['name'].endswith('-debuginfo')]
        if not src:
            rpms = [r for r in rpms if not r['arch'] == 'src']

        urls = ['%s/%s' % (baseurl, koji.pathinfo.rpm(r)) for r in rpms]
        if len(urls) == 0:
            raise exc.TaskotronRemoteError('No RPMs found for %s' % nvr)

        return sorted(urls)


    def get_nvr_rpms(self, nvr, rpm_dir, arches=None, debuginfo=False,
                     src=False):
        '''Retrieve the RPMs associated with a build NVR into the specified
            directory.

        :param str nvr build: NVR
        :param str rpm_dir: location where to store the RPMs
        :return: list of local filenames of the grabbed RPMs
        :rtype: list of str
        :raise TaskotronRemoteError: if downloading failed
        '''
        rpm_urls = self.nvr_to_urls(nvr,
                                    arches=arches,
                                    debuginfo=debuginfo,
                                    src=src)
        rpm_files = []
        log.info('Fetching RPMs for: %s', nvr)
        for url in rpm_urls:
            log.info('  fetching %s', url)
            rpm_file = file_utils.download(url, rpm_dir)
            rpm_files.append(rpm_file)

        return rpm_files


    def get_tagged_rpms(self, tag, dest, arches):
        '''Downloads all RPMs of all NVRs tagged by a specific Koji tag.

        :param str dest: destination directory
        :param arches: list of architectures
        :type arches: list of str
        :raise TaskotronRemoteError: if downloading failed
        '''
        tag_data = self.session.listTagged(tag)
        nvrs = ["%(nvr)s" % x for x in tag_data]
        rpms = []
        for nvr in nvrs:
            rpms.append(self.get_nvr_rpms(nvr, dest, arches))

        return rpms


def getNEVR(build):
    '''Extract RPM version identifier in NEVR format from Koji build object

    :param dict build: Koji buildinfo dictionary (as returned e.g. from
                       :meth:`koji.getBuild`)
    :return: NEVR string; epoch is included when non-zero, otherwise omitted
    :raise TaskotronValueError: if ``build`` is of incorrect type
    '''
    # validate input
    if (not isinstance(build, collections.Mapping) or 'nvr' not in build or
        'epoch' not in build):
        raise exc.TaskotronValueError("Input argument doesn't look like "
        "a Koji build object: %s" % build)

    rpmver = hawkey.split_nevra(build['nvr'] + '.noarch')
    rpmver.epoch = build['epoch'] or 0
    nevr = '%s-%s' % (rpmver.name, rpmver.evr())
    # supress 0 epoch (evr() method always includes epoch, even if 0)
    nevr = rpm_utils.rpmformat(nevr, fmt='nevr')

    return nevr
